local addon, core = ...

core[1] = {} -- F, functions
core[2] = {} -- C, constants/config
core[3] = {} -- L, localization

FreeUI = core

local F, C, L = unpack(select(2, ...))

C.media = {
	["backdrop"] = "Interface\\ChatFrame\\ChatFrameBackground", -- default backdrop
	["font"] = "Interface\\AddOns\\FreeUI\\media\\Hooge0655.ttf", -- default pixel font
	["glow"] = "Interface\\AddOns\\FreeUI\\media\\glowTex", -- glow/shadow texture
	["low_health"] = "Interface\\AddOns\\!FreeUI\\media\\LowHealth.mp3", -- low health sound
	["low_mana"] = "Interface\\AddOns\\FreeUI\\media\\LowMana.mp3", -- low mana sound
	["texture"] = "Interface\\AddOns\\FreeUI\\media\\statusbar", -- statusbar texture
	["totemborder"] = "Interface\\AddOns\\FreeUI\\media\\border", -- totem border (duh)
	["questborder"] = "Interface\\AddOns\\FreeUI\\media\\questglow", -- quest highlight border in bags + checked texture
}

-- [[ Functions ]]

local function dummy() end

local function CreateBD(f, a)
	f:SetBackdrop({
		bgFile = C.media.backdrop, 
		edgeFile = C.media.backdrop, 
		edgeSize = 1, 
	})
	f:SetBackdropColor(0, 0, 0, a or .5)
	f:SetBackdropBorderColor(0, 0, 0)
end

local function CreateSD(parent, size, r, g, b, alpha, offset)
	local sd = CreateFrame("Frame", nil, parent)
	sd.size = size or 5
	sd.offset = offset or 0
	sd:SetBackdrop({
		edgeFile = C.media.glow,
		edgeSize = sd.size,
	})
	sd:SetPoint("TOPLEFT", parent, -sd.size - 1 - sd.offset, sd.size + 1 + sd.offset)
	sd:SetPoint("BOTTOMRIGHT", parent, sd.size + 1 + sd.offset, -sd.size - 1 - sd.offset)
	sd:SetBackdropBorderColor(r or 0, g or 0, b or 0)
	sd:SetAlpha(alpha or 1)
end

local function CreateFS(parent, size, justify)
    local f = parent:CreateFontString(nil, "OVERLAY")
    f:SetFont(C.media.font, size, "OUTLINEMONOCHROME")
    f:SetShadowColor(0, 0, 0, 0)
    if(justify) then f:SetJustifyH(justify) end
    return f
end

local function CreatePulse(frame, speed, mult, alpha) -- pulse function originally by nightcracker
	frame.speed = speed or .05
	frame.mult = mult or 1
	frame.alpha = alpha or 1
	frame.tslu = 0 -- time since last update
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.tslu = self.tslu + elapsed
		if self.tslu > self.speed then
			self.tslu = 0
			self:SetAlpha(self.alpha)
		end
		self.alpha = self.alpha - elapsed*self.mult
		if self.alpha < 0 and self.mult > 0 then
			self.mult = self.mult*-1
			self.alpha = 0
		elseif self.alpha > 1 and self.mult < 0 then
			self.mult = self.mult*-1
		end
	end)
end

local function CreateTab(f)
	local sd = CreateFrame("Frame", nil, f)
	sd:SetBackdrop({
		bgFile = C.media.backdrop,
		edgeFile = C.media.glow,
		edgeSize = 5,
		insets = { left = 5, right = 5, top = 5, bottom = 5 },
	})
	sd:SetPoint("TOPLEFT", 6, -2)
	sd:SetPoint("BOTTOMRIGHT", -6, 0)
	sd:SetBackdropColor(0, 0, 0, .5)
	sd:SetBackdropBorderColor(0, 0, 0)
	sd:SetFrameStrata("LOW")
end

F.dummy = dummy
F.CreateBD = CreateBD
F.CreateSD = CreateSD
F.CreateFS = CreateFS
F.CreatePulse = CreatePulse
F.CreateTab = CreateTab

-- UF setup

if not roledb then roledb = {} end
if roledb.role == nil then roledb.role = 1 end