local F, C, L = unpack(select(2, ...))

local n = UnitName("player")
local lvl = UnitLevel("player")
local class = select(2, UnitClass("player"))
local realm = GetRealmName()

--[[ General config ]]

C["general"] = {
	["archaeology"] = true,			-- timer for survey cooldown
	["audio_alert"] = true, 		-- audio alert for low health/mana
	["auto_accept"] = true,			-- auto accept invites from friends and guildies
	["autorepair"] = true,			-- automatically repair items
		["autorepair_guild"] = false,		-- use guild funds for auto repairs
	["autoroll"] = true, 			-- automatically DE or greed on BoE greens (DE priority)
	["autosell"] = true,			-- automatically sell greys
	["buffreminder"] = true, 		-- reminder for selfbuffs
	["hotkey"] = false, 			-- show hot keys on buttons
	["rightbars_mouseover"] = false,	-- show right bars on mouseover (show/hide: use blizz option)
	["shapeshift"] = false,			-- enable shapeshift bar
	["tolbarad"] = true,			-- Tol barad timer on the minimap
		["tolbarad_always"] = false,		-- show timer on non 85 characters as well
	["tooltip_cursor"] = false,		-- anchor the tooltip to the cursor
}

C["unitframes"] = {
	["player"] = {"BOTTOM", UIParent, "CENTER", -275, -105},
	["target"] = {"TOP", UIParent, "CENTER", 0, -225},
	["focus"] = {"LEFT", UIParent, "LEFT", 50, 0}, 		-- only applies if playerframe_focus is false
	["pet"] = {"RIGHT", UIParent, "RIGHT", -50, 0}, 	-- only applies if playerframe_pet is false
	["party"] = {"CENTER", 280, -70}, 			-- only applies with healmode enabled
	["raid"] = {"CENTER", 280, -70}, 			-- only applies with healmode enabled

	["arena_trinkets"] = true,				-- show enemy trinket usage on arena frames
	["cast"] = {"BOTTOM", UIParent, "BOTTOM", 0, 420}, 	-- only applies with 'castbar' set to 2
	["castbar"] = 1, 					-- 1 = dps/tank cast bar, 2 = caster/healer cast bar
	["dbh_filter"] = true,					-- only highlight debuffs that you can dispel
	["healmode_oldstyle"] = false, 				-- old unit frame style for heal mode
	["playerframe_focus"] = true,				-- focus frame near player frame
	["playerframe_pet"] = true,				-- pet frame near player frame
	["raiddebuff"] = true,					-- display a debuff on party/raid frames (only when using healer setup)
}

-- [[ Profiles ]]

if lvl ~= 85 then
	C.general.autoroll = false
end

if realm == "Steamwheedle Cartel" then
	C.general.tolbarad_always = true
	C.general.autorepair_guild = true
end

if class == "MAGE" or class == "PRIEST" or class == "WARLOCK" then
	C.unitframes.castbar = 2
end

-- [[ Class modules and spells ]]

-- Class modules
C["classmod"] = {
	["rogue"] = true,		-- poison reminder
	["rogue_checkthrown"] = false,	-- check thrown weapon for poison
	["shaman"] = true, 		-- totem bars
	["warrior_shockwave"] = true,	-- alert when on 3 stacks of thunderstruck
	["warrior_shouts"] = true, 	-- shout reminder
}

-- Selfbuff reminder
C["selfbuffs"] = {
	HUNTER = {
		13165, -- hawk
		5118, -- cheetah
		13159, -- pack
		20043, -- wild
		82661, -- fox
	},
	PALADIN = {
		31801, -- seal of truth
		20154, -- seal of righteousness
		20165, -- seal of insight
		20164, -- seal of justice
	},
	PRIEST = {
		588, -- inner fire
	},
	MAGE = {
		7302, -- frost armor
		6117, -- mage armor
		30482, -- molten armor
	},
	WARLOCK = {
		28176, -- fel armor
		687, -- demon armor
	},
	SHAMAN = {
		52127, -- water shield
		324, -- lightning shield
		974, -- earth shield
	},
	DEATHKNIGHT = {
		57330, -- horn of winter
		31634, -- strength of earth
		6673, -- battle shout
		93435, -- roar of courage
	},
}

-- sFilter buff tracker
C["sfilter"] = {
	["PALADIN"] = {
	-- Divine Plea
	{spellId = 54428, size = 39, unitId = "player", isMine = 1, filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", 95, -163}},
	-- Divine Shield, Hand of Protection, Divine Protection, Avenging Wrath, Zealotry
	{spellId = 642, spellId2 = 1022, spellId3 = 498, spellId4 = 31884, spellId5 = 85696, size = 39, unitId = "player", isMine = "all", filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", 0, -163}},
	-- Inquisition
	{spellId = 84963, size = 39, unitId = "player", isMine = 1, filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", -95, -163}},
	-- Repentance
	{spellId = 20066, size = 26, unitId = "target", isMine = 1, filter = "HARMFUL", setPoint = {"BOTTOM", UIParent, "BOTTOM", 0, 140}},
	},
	["PRIEST"] = {
	-- Chakra (_, Heal, Renew, Prayer of Healing, Smite)
	{spellId = 14751, spellId2 = 81208, spellId3 = 81207, spellId4 = 81206, spellId5 = 81209, size = 39, unitId = "player", isMine = 1, filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", 0, -163}},
	},
	["ROGUE"] = {
	-- Bandit's Guile
	{spellId = 84745, spellId2 = 84746, spellId3 = 84747, size = 39, unitId = "player", isMine = 1, filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", 95, -163}},
	-- Recuperate
	{spellId = 73651, size = 39, unitId = "player", isMine = 1, filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", 95, -163}},
	-- Slice and dice
	{spellId = 5171, size = 39, unitId = "player", isMine = 1, filter = "HELPFUL", setPoint = {"CENTER", UIParent, "CENTER", -95, -163}},
	},
}

-- [[ Filters ]]

-- Debuffs you don't want to show on party/raid frames (only when raiddebuffs is true)

C["raidfilter"] = {
	["Exhaustion"] = true,
	["Forbearance"] = true,
	["Sated"] = true,
	["Temporal Displacement"] = true,
}

-- Debuffs by other players you want to show on the target frame

C["debuffFilter"] = {
	["Sunder Armor"] = true,
	["Expose Armor"] = true,
	["Thunder Clap"] = true,
	["Infected Wounds"] = true,
	["Frost Fever"] = true,
	["Demoralizing Shout"] = true,
	["Demoralizing Roar"] = true,
	["Vindication"] = true,
	["Polymorph"] = true,
	["Blind"] = true,
	["Sap"] = true,
	["Kidney Shot"] = true,
	["Cheap Shot"] = true,
	["Bash"] = true,
	["Hammer of Justice"] = true,
	["Repentance"] = true,
	["Strangulate"] = true,
	["Shackle Undead"] = true,
	["Entangling Roots"] = true,
	["Cyclone"] = true,
	["Hibernate"] = true,
	["Banish"] = true,
	["Wyvern Sting"] = true,
	["Hex"] = true,
	["Taunt"] = true,
	["Challenging Shout"] = true,
	["Mocking Blow"] = true,
	["Righteous Defense"] = true,
	["Hand of Reckoning"] = true,
	["Death Grip"] = true, 
	["Dark Command"] = true,
	["Growl"] = true,
	["Challenging Roar"] = true,
	["Bind Elemental"] = true,
	["Fear"] = true,
	["Freezing Trap"] = true,
	["Seduction"] = true,
	["Turn Evil"] = true,
}

C["dangerousBuffs"] = {
	["Adrenaline Rush"] = true,
	["Ancient Crusader"] = true,
	["Ancient Guardian"] = true,
	["Ancient Healer"] = true,
	["Ancient Hysteriya"] = true,
	["Anti-Magic Shell"] = true,
	["Ardent Defender"] = true,
	["Aura Mastery"] = true,
	["Avenging Wrath"] = true,
	["Bladestorm"] = true,
	["Bloodlust"] = true,
	["Camouflage"] = true,
	["Cloak of Shadows"] = true,
	["Dancing Rune Weapon"] = true,
	["Deterrence"] = true,
	["Dispersion"] = true,
	["Divine Protection"] = true,
	["Divine Shield"] = true,
	["Evasion"] = true,
	["Guardian Spirit"] = true,
	["Hand of Protection"] = true,
	["Heroism"] = true,
	["Ice Barrier"] = true,
	["Ice Block"] = true,
	["Icebound Fortitude"] = true,
	["Innervate"] = true,
	["Invisibility"] = true,
	["Last Stand"] = true,
	["Mage Ward"] = true,
	["Mana Shield"] = true,
	["Metamorphosis"] = true,
	["Shield Wall"] = true,
	["Spell Reflection"] = true,
	["Sprint"] = true,
	["Time Warp"] = true,
	["Pain Suppression"] = true,
	["Power Infusion"] = true,
	["Unholy Frenzy"] = true,
	["Zealotry"] = true,
}