-- rBuffFrameStyler by Roth, modified.

local F, C, L = unpack(select(2, ...))

local BuffFrame               = _G["BuffFrame"]
local TemporaryEnchantFrame   = _G["TemporaryEnchantFrame"]
local ConsolidatedBuffs       = _G["ConsolidatedBuffs"]

  ---------------------------------------
  -- FUNCTIONS
  ---------------------------------------

  --update buff anchors
  local updateBuffAnchors = function()
    --print(BUFF_ACTUAL_DISPLAY)
    local numBuffs = 0
    local buff, previousBuff, aboveBuff
    for i = 1, BUFF_ACTUAL_DISPLAY do
      buff = _G["BuffButton"..i]
      buff:SetParent(BuffFrame)
      buff.consolidated = nil
      buff.parent = BuffFrame
      buff:ClearAllPoints()
      numBuffs = numBuffs + 1
      index = numBuffs
      if ((index > 1) and (mod(index, 8) == 1)) then
        buff:SetPoint("TOP", aboveBuff, "BOTTOM", 0, -4)
        aboveBuff = buff
      elseif (index == 1) then
        buff:SetPoint("TOPRIGHT", BuffFrame, "TOPRIGHT", 0, 0)
        aboveBuff = buff
      else
        buff:SetPoint("RIGHT", previousBuff, "LEFT", -3, 0);
      end
      previousBuff = buff      
    end    
  end

  --create drag frame for temp enchant icons
  local function createTempEnchantHolder()
    local f = CreateFrame("Frame", "rBFS_TempEnchantHolder", UIParent)
    f:SetSize(50,50)
    f:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -6, 50)
  end
  createTempEnchantHolder()
  
  --create drag frame for buff icons
  local function createBuffFrameHolder()
    local f = CreateFrame("Frame", "rBFS_BuffFrameHolder", UIParent)
    f:SetSize(50,50)
    f:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -50, -50)
  end
  createBuffFrameHolder()

  --move tempenchant frame
  local moveTempEnchantFrame = function()
    local f = _G["rBFS_TempEnchantHolder"]
    TemporaryEnchantFrame:SetParent(f)
    TemporaryEnchantFrame:ClearAllPoints()
    TemporaryEnchantFrame:SetPoint("TOPRIGHT",0,0)
    for i = 1, 3 do
	local bu = _G["TempEnchant"..i]
	bu:ClearAllPoints()
	if i==1 then
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -10, 50)
	elseif i==2 then
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -10, 85)
	else
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -10, 120)
	end
    end
  end
  
  --move buff frame
  local moveBuffFrame = function()
    local f = _G["rBFS_BuffFrameHolder"]
    BuffFrame:SetParent(f)
    BuffFrame:ClearAllPoints()
    BuffFrame:SetPoint("TOPRIGHT",0,0)
  end

  --check duration
  local function durationSetText(duration, arg1, arg2)
    duration:SetText(format("|cffffffff"..string.gsub(arg1, " ", "").."|r", arg2))
  end

  --apply aura frame texture func
  local function applySkin(b,button,type)
    if b then      
      local border = _G[button.."Border"]      
      if border then border:Hide() end
      
      local icon = _G[button.."Icon"]
      icon:SetTexCoord(.08, .92, .08, .92)
      icon:SetAllPoints(b)
      icon:SetDrawLayer("BACKGROUND",-8)
      
      b.duration:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")
      b.duration:SetShadowOffset(0, 0, 0, 0)
      b.duration:ClearAllPoints()
      b.duration:SetPoint("BOTTOM", 0, -2)

      hooksecurefunc(b.duration, "SetFormattedText", durationSetText)

      b.count:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")
      b.count:ClearAllPoints()
      b.count:SetPoint("TOP", b, "TOP", 2, -2)
      
      local bg = CreateFrame("Frame", nil, b)
      bg:SetPoint("TOPLEFT", b, -1, 1)
      bg:SetPoint("BOTTOMRIGHT", b, 1, -1)
      bg:SetFrameLevel(b:GetFrameLevel()-1)        
      bg:SetBackdrop({
		edgeFile = C.media.backdrop,
		edgeSize = 1,
      })
      bg:SetBackdropBorderColor(0, 0, 0)
      
      b.styled = true
    end
  end

  --check auras func
  local function checkauras()
    --scan for buffs
    for i=1, BUFF_MAX_DISPLAY do    
      local b = _G["BuffButton"..i]    
      if b and not b.styled then
        applySkin(b,"BuffButton"..i,"buff")
      end
    end
    --scan for debuffs
    for i=1, DEBUFF_MAX_DISPLAY do
      local b = _G["DebuffButton"..i]
      if b then
        b:Hide()
        b.Show = b.Hide
      end
    end    
  end

  --init func
  local function init()
    --position buff frame
    moveBuffFrame()
    BuffFrame.SetPoint = F.dummy -- stupid ticket frame moving buffs
        
    --position temp enchant icons
    moveTempEnchantFrame()

    --skin temp enchant
    for i=1, NUM_TEMP_ENCHANT_FRAMES do
      local b = _G["TempEnchant"..i]
      if b and not b.styled then
        applySkin(b,"TempEnchant"..i, "wpn")
      end
    end

  end

  ---------------------------------------
  -- CALLS // HOOKS
  ---------------------------------------

  --hook Blizzard functions to move the buffframe
  hooksecurefunc("BuffFrame_UpdateAllBuffAnchors",    updateBuffAnchors)

  local a = CreateFrame("Frame")

  a:SetScript("OnEvent", function(self, event, ...)
    local unit = ...
    if(event=="PLAYER_ENTERING_WORLD") then
      init()
      checkauras()
    end
    if (event=="UNIT_AURA") then
      if (unit == PlayerFrame.unit) then
        checkauras()
      end
    end
  end)

  a:RegisterEvent("PLAYER_ENTERING_WORLD")
  a:RegisterEvent("UNIT_AURA")