-- Base code by Alza, most code personal.

local F, C, L = unpack(select(2, ...))

local Scale = 0.9

local _, _, _, _, date = GameTimeFrame:GetRegions()

Minimap:ClearAllPoints()
Minimap:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -50 / Scale, 50 / Scale)
Minimap:SetMaskTexture("Interface\\Buttons\\WHITE8X8")
MinimapCluster:SetScale(Scale)

local f = CreateFrame("Frame", nil, Minimap)
f:SetBackdrop({
	edgeFile = C.media.backdrop, 
	edgeSize = 1 / Scale, 
})
f:SetPoint("TOPLEFT", Minimap, -1 / Scale, 1 / Scale)
f:SetPoint("BOTTOMRIGHT", Minimap, 1 / Scale, -1 / Scale)
f:SetBackdropBorderColor(0, 0, 0)
f:SetFrameStrata("BACKGROUND")

FreeUIMailFrame = CreateFrame("Frame", FreeUIMailFrame, Minimap)
local mail = FreeUIMailFrame
mail:RegisterEvent("UPDATE_PENDING_MAIL")
mail:SetScript("OnEvent", function()
	if HasNewMail() then
		mail:Show()
	else
		mail:Hide()
	end
end)
mail:Hide()

local mt = F.CreateFS(mail, 8 / Scale)
mt:SetPoint("BOTTOM", Minimap, 0, 6)
mt:SetText("Mail")

ZoneTextString:ClearAllPoints()
ZoneTextString:SetPoint("CENTER", Minimap)
ZoneTextString:SetWidth(Minimap:GetWidth())
ZoneTextString:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")
SubZoneTextString:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")
PVPInfoTextString:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")
PVPArenaTextString:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")

local bu = MinimapZoneTextButton
local text = MinimapZoneText

bu:ClearAllPoints()
bu:SetPoint("CENTER", Minimap)
bu:SetFrameStrata("HIGH")
bu:EnableMouse(false)
bu:SetAlpha(0)
text:SetFont(C.media.font, 8 / Scale, "OUTLINEMONOCHROME")
text:SetPoint("CENTER", bu)
text:SetShadowColor(0, 0, 0, 0)
text:SetJustifyH("CENTER")

Minimap:HookScript("OnEnter", function()
	bu:SetAlpha(1)
end)
Minimap:HookScript("OnLeave", function()
	bu:SetAlpha(0)
end)

local function CaptureBar()
	for i = 1, NUM_EXTENDED_UI_FRAMES do
		local bar = _G["WorldStateCaptureBar"..i]
		if bar and bar:IsShown() then
			bar:ClearAllPoints()
			bar:SetPoint("TOP", UIParent, "TOP", 0, -120)
		end
	end
end

hooksecurefunc("WorldStateAlwaysUpFrame_Update", CaptureBar)

local styled = false

local function StateText()
	for i = 1, 3 do
		local f = _G["AlwaysUpFrame"..i]
		if f and not f.styled then
			local _, g = f:GetRegions()
			g:SetFont(C.media.font, 8, "OUTLINEMONOCHROME")
			g:SetShadowColor(0, 0, 0, 0)
			g:SetTextColor(1, 1, 1)
			f.styled = true
		end
	end
end

hooksecurefunc("WorldStateAlwaysUpFrame_Update", StateText)

do
	local frames = {
		"GuildInstanceDifficulty",
		"MiniMapInstanceDifficulty",
		"MiniMapBattlefieldBorder",
		"MiniMapVoiceChatFrame",
		"MiniMapWorldMapButton",
		"MiniMapMailFrame",
		"MiniMapMailBorder",
		"MinimapBorderTop",
		"MinimapNorthTag",
		"MiniMapTracking",
		"MinimapZoomOut",
		"MinimapZoomIn",
		"MinimapBorder",
	}

	for i = 1, #frames do
		_G[frames[i]]:Hide()
		_G[frames[i]].Show = F.dummy
	end
end

Minimap:EnableMouseWheel(true)
Minimap:SetScript("OnMouseWheel", function(_, zoom)
	if(zoom>0) then
		Minimap_ZoomIn()
	else
		Minimap_ZoomOut()
	end
end)

Minimap:SetScript("OnMouseUp", function(self, button)
	if(button=="RightButton") then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, "cursor", 0, 0)
	elseif(button=="MiddleButton") then
		if(not CalendarFrame) then LoadAddOn("Blizzard_Calendar") end
		Calendar_Toggle()
	else
		Minimap_OnClick(self)
	end
end)

MinimapCluster:EnableMouse(false)

GameTimeFrame:ClearAllPoints()
GameTimeFrame:SetPoint("TOPRIGHT", Minimap, "TOPRIGHT", -2, -2)

local bu = _G["GameTimeFrame"]
bu:SetWidth(24)
bu:SetHeight(24)
bu:SetHitRectInsets(0, 0, 0, 0)

date:SetTextColor(1, 1, 1)
date:SetFont(C.media.font, 8 / Scale, "OUTLINEMONOCHROME")
date:ClearAllPoints()
date:SetPoint("CENTER", bu, "CENTER")

bu:SetNormalTexture("")
bu:SetPushedTexture("")
bu:SetHighlightTexture("")

MiniMapBattlefieldFrame:ClearAllPoints()
MiniMapBattlefieldFrame:SetPoint("TOP", Minimap, "TOP", 0, 3)

MiniMapLFGFrameBorder:SetAlpha(0)
MiniMapLFGFrame:ClearAllPoints()
MiniMapLFGFrame:SetPoint("BOTTOMRIGHT", Minimap)

TicketStatusFrame:ClearAllPoints()
TicketStatusFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -49, 0)
F.CreateBD(TicketStatusFrameButton)

LFDSearchStatus:ClearAllPoints()
LFDSearchStatus:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMLEFT", -4.5, -1.5)

StreamingIcon:ClearAllPoints()
StreamingIcon:SetPoint("BOTTOMLEFT", Minimap, "BOTTOMLEFT")

TimeManagerClockButton:Hide()

local rd = CreateFrame("Frame", nil, Minimap)
rd:SetPoint("TOPLEFT", Minimap, "TOPLEFT", 0, -2)
rd:RegisterEvent("PLAYER_ENTERING_WORLD")
rd:RegisterEvent("PLAYER_DIFFICULTY_CHANGED")
rd:RegisterEvent("GUILD_PARTY_STATE_UPDATED")
rd:SetSize(30, 22)

local rdt = F.CreateFS(rd, 8 / Scale)
rdt:SetPoint("CENTER")

local function diff()
	local inInstance, instanceType = IsInInstance()
	local _, _, difficultyIndex, _, _, dynamicDifficulty, isDynamic = GetInstanceInfo()

	if inInstance and instanceType == "raid" then
		if (isDynamic and difficultyIndex == 1 and dynamicDifficulty == 0) or (not isDynamic and difficultyIndex == 1) then
			rdt:SetText("10")
		elseif (isDynamic and (difficultyIndex == 3 and dynamicDifficulty == 0) or (difficultyIndex == 1 and dynamicDifficulty == 1)) or (not isDynamic and difficultyIndex == 3) then
			rdt:SetText("10H")
		elseif (isDynamic and difficultyIndex == 2 and dynamicDifficulty == 0) or (not isDynamic and difficultyIndex == 2) then
			rdt:SetText("25")
		elseif (isDynamic and (difficultyIndex == 2 and dynamicDifficulty == 1) or (difficultyIndex == 4)) or (not isDynamic and difficultyIndex == 4) then
			rdt:SetText("25H")
		end
	elseif inInstance and instanceType == "party" then
		if difficultyIndex == 1 then
			rdt:SetText("5")
		elseif difficultyIndex == 2 then
			rdt:SetText("5H")
		end
	else
		rdt:SetText("")
	end
end

rd:SetScript("OnEvent", diff)