local F, C, L = unpack(select(2, ...))

local parent, ns = ...
local oUF = ns.oUF

local class = select(2, UnitClass("player"))

local colors = setmetatable({
	power = setmetatable({
		["MANA"] = {.9, .9, .9},
		["RAGE"] = {.9, .1, .1},
		["FUEL"] = {0, 0.55, 0.5},
		["FOCUS"] = {.9, .5, .1},
		["ENERGY"] = {.9, .9, .1},
		["AMMOSLOT"] = {0.8, 0.6, 0},
		["RUNIC_POWER"] = {.1, .9, .9},
		["POWER_TYPE_STEAM"] = {0.55, 0.57, 0.61},
		["POWER_TYPE_PYRITE"] = {0.60, 0.09, 0.17},
	}, {__index = oUF.colors.power}),
	class = setmetatable({
		["DEATHKNIGHT"] = {0.77, 0.12, 0.23},
		["DRUID"] = {1, 0.49, 0.04},
		["HUNTER"] = {0.58, 0.86, 0.49},
		["MAGE"] = {0, 0.76, 1},
		["PALADIN"] = {1, 0.22, 0.52},
		["PRIEST"] = {0.8, 0.87, .9},
		["ROGUE"] = {1, 0.91, 0.2},
		["SHAMAN"] = {0, 0.6, 0.6},
		["WARLOCK"] = {0.6, 0.47, 0.85},
		["WARRIOR"] = {0.9, 0.65, 0.45},
	}, {__index = oUF.colors.class}),
}, {__index = oUF.colors})

--[[ Dropdown menu ]]

-- from oUF_Lily

local name, addon = ...
local dropdown = CreateFrame('Frame', 'oUF_FreeDropDown', UIParent, 'UIDropDownMenuTemplate')

function addon:menu()
	dropdown:SetParent(self)
	return ToggleDropDownMenu(1, nil, dropdown, 'cursor', 0, 0)
end

-- Slightly altered version of:
-- FrameXML/CompactUnitFrame.lua:730:CompactUnitFrameDropDown_Initialize
local init = function(self)
	local unit = self:GetParent().unit
	local menu, name, id

	if(not unit) then
		return
	end

	if(UnitIsUnit(unit, "player")) then
		menu = "SELF"
	elseif(UnitIsUnit(unit, "vehicle")) then
		-- NOTE: vehicle check must come before pet check for accuracy's sake because
		-- a vehicle may also be considered your pet
		menu = "VEHICLE"
	elseif(UnitIsUnit(unit, "pet")) then
		menu = "PET"
	elseif(UnitIsPlayer(unit)) then
		id = UnitInRaid(unit)
		if(id) then
			menu = "RAID_PLAYER"
			name = GetRaidRosterInfo(id)
		elseif(UnitInParty(unit)) then
			menu = "PARTY"
		else
			menu = "PLAYER"
		end
	else
		menu = "TARGET"
		name = RAID_TARGET_ICON
	end

	if(menu) then
		UnitPopup_ShowMenu(self, menu, unit, name, id)
	end
end

UIDropDownMenu_Initialize(dropdown, init, 'MENU')

--[[ Short values ]]

local siValue = function(val)
	if(val >= 1e6) then
		return format("%.2fm", val * 0.000001)
	elseif(val >= 1e4) then
		return format("%.1fk", val * 0.001) 
	else
		return val
	end
end

--[[ Tags ]]

oUF.Tags['free:health'] = function(unit)
	if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	if(unit=="target" or(unit and unit:find("boss%d"))) then
		return format("|cffffffff%s|r %.0f", siValue(min), (min/max)*100)
	else
		return siValue(min)
	end
end
oUF.TagEvents['free:health'] = oUF.TagEvents.missinghp

oUF.Tags['free:maxhealth'] = function(unit)
	if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	local max = UnitHealthMax(unit)
	return max
end
oUF.TagEvents['free:maxhealth'] = oUF.TagEvents.missinghp

oUF.Tags['free:missinghealth'] = function(unit)
	local min, max, name = UnitHealth(unit), UnitHealthMax(unit), UnitName(unit)
	if name and name:len() > 4 then name = name:sub(1, 4) end

	if not UnitIsConnected(unit) then
		return "Off" 
	elseif UnitIsDead(unit) then
		return "Dead"
	elseif UnitIsGhost(unit) then 
		return "Ghost"
	elseif min ~= max then
		return siValue(max-min)
	else
		return name
	end
end
oUF.TagEvents['free:missinghealth'] = oUF.TagEvents.missinghp

oUF.Tags['free:power'] = function(unit)
	local min, max = UnitPower(unit), UnitPowerMax(unit)
	if(min == 0 or max == 0 or not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	return siValue(min)
end
oUF.TagEvents['free:power'] = oUF.TagEvents.missingpp

--[[ Update health bar colour ]]

local UpdateHealth = function(self, event, unit)
	if(self.unit == unit) then
		local r, g, b, t
		local min, max
		if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit)) then
			r, g, b = .6, .6, .6
		elseif(UnitIsPlayer(unit)) then
			local _, class = UnitClass(unit)
			t = self.colors.class[class]
		elseif(unit and unit:find("boss%d")) then
			min, max = UnitHealth(unit), UnitHealthMax(unit)
			r, g, b = self.ColorGradient(min / max, unpack(self.colors.smooth))
		else
			t = C.reactioncolours[UnitReaction(unit, "player")]
		end

		if(t) then
			r, g, b = t[1], t[2], t[3]
		end

		if(r) then
			if roledb.role == 2 and C.unitframes.healmode_oldstyle == false then
				self.Health:SetStatusBarColor(0, 0, 0, 0)
				self.Power:SetStatusBarColor(r, g, b)
			else
				self.Health:SetStatusBarColor(r, g, b)
			end
		end

		if roledb.role == 2 and C.unitframes.healmode_oldstyle == false then
			min, max = UnitHealth(unit), UnitHealthMax(unit)
			self.Healthdef:SetPoint("LEFT", self.Health, self:GetWidth() * (min/max), 0)
			if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then
				self.Healthdef:Hide()
				if self.Text then self.Text:SetTextColor(.3, .7, 1) end
			else
				self.Healthdef:Show()
				self.Healthdef:SetVertexColor(self.ColorGradient(min / max, unpack(self.colors.smooth)))
				if self.Text then self.Text:SetTextColor(1, 1, 1) end
			end
		end
	end
end

--[[ Update health ]]

local PostUpdateHealth = function(Health, unit, min, max)
	if(UnitIsDead(unit)) then
		Health:SetValue(0)
	elseif(UnitIsGhost(unit)) then
		Health:SetValue(0)
	end

	if(Health.value) then
		if(unit=="target") then
			Health.value:SetTextColor(unpack(C.reactioncolours[UnitReaction("player", unit) or 5]))
		end
	end

	return UpdateHealth(Health:GetParent(), 'PostUpdateHealth', unit)
end

--[[ Hide Blizz frames ]]

CompactRaidFrameManagerDisplayFrameOptionsButton:Hide()
CompactRaidFrameManagerToggleButton:SetWidth(0.001)
CompactRaidFrameManager:SetWidth(0.001)

--[[ Aura icon filter ]]

local PostUpdateIcon
do
	local playerUnits = {
		player = true,
		pet = true,
		vehicle = true,
	}

	PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
		local texture = icon.icon
		local name, _, _, _, dtype, duration, expirationTime, unitCaster = UnitAura(unit, index, icon.filter)
		if(unit~="target") then icon:Show() end
		if(not playerUnits[icon.owner] and not C.debuffFilter[name] and not UnitIsFriend("player", unit) and icon.debuff) or(UnitIsPlayer(unit) and not UnitIsFriend("player", unit) and not icon.debuff and not C.dangerousBuffs[name]) then
			icon:Hide()
		else
			icon:Show()
		end
	end
end

--[[ Update power value ]]

local PostUpdatePower = function(Power, unit, min, max)
	local Health = Power:GetParent().Health
	if(min == 0 or max == 0 or not UnitIsConnected(unit)) then
		Power:SetValue(0)
	elseif(UnitIsDead(unit) or UnitIsGhost(unit)) then
		Power:SetValue(0)
	end
end

local RAID_TARGET_UPDATE = function(self, event)
	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		self.RIcon:SetText(ICON_LIST[index].."22|t")
	else
		self.RIcon:SetText()
	end
end

--[[ Global ]]

local Shared = function(self, unit, isSingle)
	self.menu = addon.menu

	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	self:RegisterForClicks("AnyUp")

	self.bd = CreateFrame("Frame", nil, self)
	self.bd:SetPoint("TOPLEFT", self, -1, 1)
	self.bd:SetPoint("BOTTOMRIGHT", self, 1, -1)
	self.bd:SetFrameStrata("BACKGROUND")
	F.CreateBD(self.bd)

	self.bg = CreateFrame("Frame", nil, self)
	self.bg:SetPoint("TOPLEFT", self, -6, 6)
	self.bg:SetPoint("BOTTOMRIGHT", self, 6, -6)
	self.bg:SetFrameStrata("BACKGROUND")
	self.bg:SetBackdrop({
		edgeFile = C.media.glow,
		edgeSize = 5,
	})
	self.bg:SetBackdropBorderColor(0, 0, 0, 0)

	local dbh = CreateFrame("Frame")
	self.DebuffHighlight = dbh
	self.DebuffHighlightFrame = true

	if C.unitframes.dbh_filter == true then
		self.DebuffHighlightFilter = true
	else
		self.DebuffHighlightFilter = false
	end

	--[[ Health ]]

	local Health = CreateFrame("StatusBar", nil, self)
	Health:SetStatusBarTexture(C.media.texture)
	Health:SetFrameStrata("LOW")

	Health.frequentUpdates = true

	Health:SetPoint("TOP")
	Health:SetPoint("LEFT")
	Health:SetPoint("RIGHT")
	Health:SetPoint("BOTTOM", 0, 2)

	self.Health = Health

	--[[ Health deficit colour (thanks nin) ]]

	if roledb.role == 2 and C.unitframes.healmode_oldstyle == false then
		local Healthdef = Health:CreateTexture(nil, "BORDER")
		Healthdef:SetTexture(C.media.texture)
		Healthdef:SetPoint("TOPRIGHT", Health)
		Healthdef:SetPoint("BOTTOMRIGHT", Health)
		Healthdef:SetPoint("LEFT", Health, 0, 0)

		self.Healthdef = Healthdef
	end

	--[[ Power ]]

	local Power = CreateFrame("StatusBar", nil, self)
	Power:SetStatusBarTexture(C.media.texture)
	Power:SetFrameStrata("LOW")

	if roledb.role == 2 and C.unitframes.healmode_oldstyle == false then Power.colorPower = false else Power.colorPower = true end
	Power.frequentUpdates = true

	Power:SetHeight(1)

	Power:SetPoint("LEFT")
	Power:SetPoint("RIGHT")
	Power:SetPoint("TOP", Health, "BOTTOM", 0, -1)

	self.Power = Power

	local Powerbg = CreateFrame("Frame", nil, self)
	Powerbg:SetPoint("TOPLEFT", self.Power, -1, 1)
	Powerbg:SetPoint("BOTTOMRIGHT", self.Power, 1, -1)
	Powerbg:SetFrameStrata("BACKGROUND")
	F.CreateBD(Powerbg, 0)
	local _, powertype = UnitPowerType("player")
	if powertype == "RAGE" then
		Powerbg:SetBackdropColor(0, 0, 0, .3)
	end

	--[[ Alt Power ]]

	local AltPowerBar = CreateFrame("StatusBar", nil, self)
	AltPowerBar:SetWidth(200)
	AltPowerBar:SetHeight(10)
	AltPowerBar:SetStatusBarTexture(C.media.texture)
	AltPowerBar:SetStatusBarColor(unpack(C.class))

	AltPowerBar:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 150)

	local abd = CreateFrame("Frame", nil, AltPowerBar)
	abd:SetPoint("TOPLEFT", -1, 1)
	abd:SetPoint("BOTTOMRIGHT", 1, -1)
	abd:SetFrameLevel(AltPowerBar:GetFrameLevel()-1)
	F.CreateBD(abd)

	self.AltPowerBar = AltPowerBar

	--[[ Castbar ]]

	local Castbar = CreateFrame("StatusBar", nil, self)
	Castbar:SetStatusBarTexture("")

	local Spark = Castbar:CreateTexture(nil, "OVERLAY")
	Spark:SetTexture"Interface\\CastingBar\\UI-CastingBar-Spark"
	Spark:SetBlendMode"ADD"
	Spark:SetWidth(16)
	Castbar.Spark = Spark

	self.Castbar = Castbar

	local PostCastStart = function(Castbar, unit, spell, spellrank)
		if Castbar.Text then Castbar.Text:SetText(spell) end

		if self.Iconbg then
			if Castbar.interrupt and unit=="target" then
				self.Iconbg:SetBackdropBorderColor(1, 0, 0)
			else
				self.Iconbg:SetBackdropBorderColor(0, 0, 0)
			end
		end
	end

	local PostCastStop = function(Castbar, unit)
		if Castbar.Text then Castbar.Text:SetText("") end
	end

	local PostCastStopUpdate = function(self, event, unit)
		if(unit ~= self.unit) then return end
		return PostCastStop(self.Castbar, unit)
	end

	-- [[ Heal prediction ]]

	if roledb.role == 2 then
		local mhpb = CreateFrame("StatusBar", nil, self.Health)
		mhpb:SetPoint("TOPLEFT", self.Health:GetStatusBarTexture(), "TOPRIGHT", 0, 0)
		mhpb:SetPoint("BOTTOMLEFT", self.Health:GetStatusBarTexture(), "BOTTOMRIGHT", 0, 0)
		mhpb:SetStatusBarTexture(C.media.texture)
		mhpb:SetStatusBarColor(0, .5, 1, 0.75)

		local ohpb = CreateFrame("StatusBar", nil, self.Health)
		ohpb:SetPoint("TOPLEFT", mhpb:GetStatusBarTexture(), "TOPRIGHT", 0, 0)
		ohpb:SetPoint("BOTTOMLEFT", mhpb:GetStatusBarTexture(), "BOTTOMRIGHT", 0, 0)
		ohpb:SetStatusBarTexture(C.media.texture)
		ohpb:SetStatusBarColor(.5, 0, 1, 0.75)

		if unit == "player" or unit == "target" then
			mhpb:SetWidth(229)
			ohpb:SetWidth(229)
		elseif unit == "pet" or unit == "focus" then
			mhpb:SetWidth(113)
			ohpb:SetWidth(113)
		else
			mhpb:SetWidth(42)
			ohpb:SetWidth(42)
		end

		self.mhpb = mhpb
		self.ohpb = ohpb

		self.HealPrediction = {
			-- status bar to show my incoming heals
			myBar = mhpb,

			-- status bar to show other peoples incoming heals
			otherBar = ohpb,

			-- amount of overflow past the end of the health bar
			maxOverflow = 1,
		}
	end

	--[[ Set up the layout ]]

	self.colors = colors

	self.disallowVehicleSwap = true

	if(isSingle) then
		if(unit=="player" or unit=="target") then
			self:SetSize(229, 14)
		elseif(unit=="pet" or unit=="focus") then
			self:SetSize(113, 8)
		elseif(unit and(unit:find("boss%d") or unit:find("arena%d"))) then
			self:SetSize(170, 14)
		end
	end

	self:RegisterEvent("UNIT_NAME_UPDATE", PostCastStopUpdate)
	table.insert(self.__elements, PostCastStopUpdate)

	Castbar.PostChannelStart = PostCastStart
	Castbar.PostCastStart = PostCastStart

	Castbar.PostCastStop = PostCastStop
	Castbar.PostChannelStop = PostCastStop

	Health.PostUpdate = PostUpdateHealth
	Power.PostUpdate = PostUpdatePower
end

-- [[ Unit specific functions ]]

local UnitSpecific = {
	pet = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(6)

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())
	end,

	player = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(12)

		local HealthPoints = F.CreateFS(Health, 8)
		local MaxHealthPoints = F.CreateFS(Health, 8, "RIGHT")

		HealthPoints:SetPoint("LEFT", self, "TOPLEFT", 0, 6)
		HealthPoints:SetJustifyH("LEFT")
		MaxHealthPoints:SetPoint("RIGHT", self, "TOPRIGHT", 0, 6)

		self:Tag(HealthPoints, '[dead][offline][free:health]')
		self:Tag(MaxHealthPoints, '[free:maxhealth]')
		Health.value = HealthPoints

		local _, UnitPowerType = UnitPowerType("player")
		if(UnitPowerType=="MANA") then
			local PowerPoints = F.CreateFS(Power, 8)
			PowerPoints:SetPoint("LEFT", HealthPoints, "RIGHT", 2, 0)
			PowerPoints:SetTextColor(.4, .7, 1)

			self:Tag(PowerPoints, '[free:power]')
			Power.value = PowerPoints
		end

		if class == "DEATHKNIGHT" then
			local runes = CreateFrame("Frame", nil, self)
			runes:SetWidth(229)
			runes:SetHeight(2)
			runes:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)

			local rbd = CreateFrame("Frame", nil, runes)
			rbd:SetBackdrop({
				edgeFile = C.media.backdrop,
				edgeSize = 1,
			})
			rbd:SetBackdropBorderColor(0, 0, 0)
			rbd:SetPoint("TOPLEFT", runes, -1, 1)
			rbd:SetPoint("BOTTOMRIGHT", runes, 1, -1)

			for i = 1, 6 do
				runes[i] = CreateFrame("StatusBar", nil, self)
				runes[i]:SetHeight(2)
				runes[i]:SetStatusBarTexture(C.media.texture)
				runes[i]:SetStatusBarColor(255/255,101/255,101/255)

				local rbd = CreateFrame("Frame", nil, runes[i])
				rbd:SetBackdrop({
					edgeFile = C.media.backdrop,
					edgeSize = 1,
				})
				rbd:SetBackdropBorderColor(0, 0, 0)
				rbd:SetPoint("TOPLEFT", runes[i], -1, 1)
				rbd:SetPoint("BOTTOMRIGHT", runes[i], 1, -1)

				if i == 1 then
					runes[i]:SetPoint("LEFT", runes)
					runes[i]:SetWidth(38)
				elseif i == 2 then
					runes[i]:SetPoint("LEFT", runes[i-1], "RIGHT", 1, 0)
					runes[i]:SetWidth(38)
				else
					runes[i]:SetPoint("LEFT", runes[i-1], "RIGHT", 1, 0)
					runes[i]:SetWidth(37)
				end
			end

			self.Runes = runes
		elseif class == "DRUID" then
			local eclipseBar = CreateFrame("Frame", nil, self)
			eclipseBar:SetWidth(229)
			eclipseBar:SetHeight(2)
			eclipseBar:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)

			local ebd = CreateFrame("Frame", nil, eclipseBar)
			ebd:SetBackdrop({
				edgeFile = C.media.backdrop,
				edgeSize = 1,
			})
			ebd:SetBackdropBorderColor(0, 0, 0)
			ebd:SetPoint("TOPLEFT", eclipseBar, -1, 1)
			ebd:SetPoint("BOTTOMRIGHT", eclipseBar, 1, -1)
			ebd:SetFrameStrata("BACKGROUND")

			local glow = CreateFrame("Frame", nil, eclipseBar)
			glow:SetBackdrop({
				edgeFile = C.media.glow,
				edgeSize = 5,
			})
			glow:SetPoint("TOPLEFT", eclipseBar, -6, 6)
			glow:SetPoint("BOTTOMRIGHT", eclipseBar, 6, -6)

			local hasEclipse = function(self, unit)
				if self.hasSolarEclipse then
					glow:SetBackdropBorderColor(.30, .52, .90, 1)
				elseif self.hasLunarEclipse then
					glow:SetBackdropBorderColor(.80, .82, .60, 1)
				else
					glow:SetBackdropBorderColor(0, 0, 0, 0)
				end
			end 

			local LunarBar = CreateFrame("StatusBar", nil, eclipseBar)
			LunarBar:SetPoint("LEFT", eclipseBar, "LEFT", 0, 0)
			LunarBar:SetSize(eclipseBar:GetWidth(), eclipseBar:GetHeight())
			LunarBar:SetStatusBarTexture(C.media.texture)
			LunarBar:SetStatusBarColor(.30, .52, .90)
			LunarBar:SetFrameStrata("LOW")
			eclipseBar.LunarBar = LunarBar

			local SolarBar = CreateFrame("StatusBar", nil, eclipseBar)
			SolarBar:SetPoint("LEFT", LunarBar:GetStatusBarTexture(), "RIGHT", 0, 0)
			SolarBar:SetSize(eclipseBar:GetWidth(), eclipseBar:GetHeight())
			SolarBar:SetStatusBarTexture(C.media.texture)
			SolarBar:SetStatusBarColor(.80, .82, .60)
			SolarBar:SetFrameStrata("LOW")
			eclipseBar.SolarBar = SolarBar

			local eclipseBarText = F.CreateFS(eclipseBar, 24)
			eclipseBarText:SetPoint("LEFT", self, "RIGHT", 10, 0)
			self:Tag(eclipseBarText, '[pereclipse]')

			self.EclipseBar = eclipseBar

			self.EclipseBar.PostUnitAura = hasEclipse

			self.EclipseBar.PostUpdatePower = function(self)
				if GetEclipseDirection() == "sun" then
					eclipseBarText:SetTextColor(.80, .82, .60)
				elseif GetEclipseDirection() == "moon" then
					eclipseBarText:SetTextColor(.30, .52, .90)
				else
					eclipseBarText:SetTextColor(1, 1, 1)
				end
			end
		elseif class == "PALADIN" then
			local UpdateHoly = function(self, event, unit, powerType)
				if(self.unit ~= unit or (powerType and powerType ~= 'HOLY_POWER')) then return end
				local num = UnitPower(unit, SPELL_POWER_HOLY_POWER)
				for i = 1, MAX_HOLY_POWER do
					if(i <= num) then
						self.glow:SetAlpha(1)
						F.CreatePulse(self.glow)
						self.count:SetText(num)
						self.count:SetTextColor(1, 1, 0)
						self.count:SetFont(C.media.font, 40, "OUTLINEMONOCHROME")
					elseif num == 0 then
						self.glow:SetScript("OnUpdate", nil)
						self.glow:SetAlpha(0)
						self.count:SetText("")
					else
						self.glow:SetScript("OnUpdate", nil)
						self.glow:SetAlpha(0)
						self.count:SetText(num)
						self.count:SetTextColor(1, 1, 1)
						self.count:SetFont(C.media.font, 24, "OUTLINEMONOCHROME")
					end
				end
			end

			local glow = CreateFrame("Frame", nil, self)
			glow:SetBackdrop({
				edgeFile = C.media.glow,
				edgeSize = 5,
			})
			glow:SetPoint("TOPLEFT", self, -6, 6)
			glow:SetPoint("BOTTOMRIGHT", self, 6, -6)
			glow:SetBackdropBorderColor(228/255,225/255,16/255)

			self.glow = glow

			local count = F.CreateFS(self, 24)
			count:SetPoint("LEFT", self, "RIGHT", 10, 0)

			self.count = count

			self.HolyPower = glow
			glow.Override = UpdateHoly
		elseif class == "WARLOCK" then
			local bars = CreateFrame("Frame", nil, self)
			bars:SetWidth(229)
			bars:SetHeight(1)
			bars:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)

			local bbd = CreateFrame("Frame", nil, bars)
			bbd:SetBackdrop({
				bgFile = C.media.backdrop,
				edgeFile = C.media.backdrop,
				edgeSize = 1,
			})
			bbd:SetBackdropColor(0, 0, 0, .5)
			bbd:SetBackdropBorderColor(0, 0, 0)
			bbd:SetPoint("TOPLEFT", bars, -1, 1)
			bbd:SetPoint("BOTTOMRIGHT", bars, 1, -1)
			bbd:SetFrameStrata("BACKGROUND")

			for i = 1, 3 do
				bars[i] = CreateFrame("StatusBar", nil, self)
				bars[i]:SetHeight(1)
				bars[i]:SetStatusBarTexture(C.media.texture)
				bars[i]:SetStatusBarColor(255/255,101/255,101/255)

				local bbd = CreateFrame("Frame", nil, bars[i])
				bbd:SetBackdrop({
					edgeFile = C.media.backdrop,
					edgeSize = 1,
				})
				bbd:SetBackdropBorderColor(0, 0, 0)
				bbd:SetPoint("TOPLEFT", bars[i], -1, 1)
				bbd:SetPoint("BOTTOMRIGHT", bars[i], 1, -1)

				if i == 1 then
					bars[i]:SetPoint("LEFT", bars)
					bars[i]:SetWidth(75)
				else
					bars[i]:SetPoint("LEFT", bars[i-1], "RIGHT", 1, 0)
					bars[i]:SetWidth(76)
				end
			end

			self.SoulShards = bars
		end

		Castbar.Width = self:GetWidth()
		Spark:SetHeight(self.Health:GetHeight())
		Castbar.Text = F.CreateFS(Castbar, 8)

		local IconFrame = CreateFrame("Frame", nil, Castbar)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
		self.Iconbg:SetBackdrop({
			edgeFile = C.media.backdrop, 
			edgeSize = 1, 
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameLevel(IconFrame:GetFrameLevel()-1)

		if C.unitframes.castbar == 2 then
			Castbar:SetStatusBarTexture(C.media.texture)
			Castbar:SetStatusBarColor(unpack(C.class))
			Castbar:SetWidth(self:GetWidth())
			Castbar:SetHeight(self:GetHeight())
			Castbar:SetPoint(unpack(C.unitframes.cast))
			Castbar.Text:SetAllPoints(Castbar)
			local sf = Castbar:CreateTexture(nil, "OVERLAY")
			sf:SetVertexColor(.5, .5, .5, .5)
			Castbar.SafeZone = sf
			IconFrame:SetPoint("LEFT", Castbar, "RIGHT", 3, 0)
			IconFrame:SetSize(22, 22)

			local bg = CreateFrame("Frame", nil, Castbar)
			bg:SetBackdrop({
				bgFile = C.media.backdrop,
				edgeFile = C.media.backdrop, 
				edgeSize = 1, 
			})
			bg:SetPoint("TOPLEFT", Castbar, -1, 1)
			bg:SetPoint("BOTTOMRIGHT", Castbar, 1, -1)
			bg:SetBackdropColor(0, 0, 0, .5)
			bg:SetBackdropBorderColor(0, 0, 0)
			bg:SetFrameStrata("BACKGROUND")
		else
			Castbar:SetAllPoints(Health)
			Castbar.Text:SetAllPoints(Health)
			IconFrame:SetPoint("RIGHT", self, "LEFT", -10, 0)
			IconFrame:SetSize(44, 44)
		end

		local RaidIcon = F.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Debuffs = CreateFrame("Frame", nil, self)
		Debuffs.initialAnchor = "TOPRIGHT"
		if class == "DEATHKNIGHT" or class == "DRUID" or class == "WARLOCK" then
			Debuffs:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -9)
		else
			Debuffs:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -4)
		end
		Debuffs["growth-x"] = "LEFT"
		Debuffs["growth-y"] = "DOWN"
		Debuffs['spacing-x'] = 3
		Debuffs['spacing-y'] = 3

		Debuffs:SetHeight(60)
		Debuffs:SetWidth(229)
		Debuffs.num = 8
		Debuffs.size = 26

		self.Debuffs = Debuffs

		Debuffs.PostUpdateIcon = PostUpdateIcon
	end,

	target = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(12)

		local HealthPoints = F.CreateFS(Health, 8)

		HealthPoints:SetPoint("LEFT", self, "TOPLEFT", 0, 6)
		HealthPoints:SetJustifyH("LEFT")

		self:Tag(HealthPoints, '[dead][offline][free:health]')
		Health.value = HealthPoints

		local PowerPoints = F.CreateFS(Power, 8)
		PowerPoints:SetPoint("BOTTOMLEFT", Health.value, "BOTTOMRIGHT")

		self:Tag(PowerPoints, '[free:power]')

		Power.value = PowerPoints

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		Castbar.Text = F.CreateFS(Health, 8)
		Castbar.Text:SetAllPoints(Health)

		local IconFrame = CreateFrame("Frame", nil, Castbar)
		IconFrame:SetPoint("LEFT", self, "RIGHT", 3, 0)
		IconFrame:SetHeight(44)
		IconFrame:SetWidth(44)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
		self.Iconbg:SetBackdrop({
			edgeFile = C.media.backdrop, 
			edgeSize = 1, 
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameLevel(IconFrame:GetFrameLevel()-1)

		local Name = F.CreateFS(Health, 8)
		Name:SetPoint("BOTTOMLEFT", Power.value, "BOTTOMRIGHT")
		Name:SetPoint("RIGHT", self)
		Name:SetJustifyH"RIGHT"
		Name:SetTextColor(1, 1, 1)

		self:Tag(Name, '[name]')
		self.Name = Name

		local RaidIcon = F.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -4)
		Auras.initialAnchor = "TOPLEFT"
		Auras["growth-x"] = "RIGHT"
		Auras["growth-y"] = "DOWN"
		Auras['spacing-x'] = 3
		Auras['spacing-y'] = 3
		Auras.numDebuffs = 16
		Auras.numBuffs = 16
		Auras:SetHeight(500)
		Auras:SetWidth(229)
		Auras.size = 26
		Auras.gap = true

		self.Auras = Auras

		Auras.PostUpdateIcon = PostUpdateIcon
	end,

	focus = function(self, ...)
		Shared(self, ...)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(6)

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(Health:GetHeight())

		local Debuffs = CreateFrame("Frame", nil, self)
		Debuffs:SetPoint("CENTER", UIParent, "CENTER", -216, -56)
		Debuffs.initialAnchor = "BOTTOMLEFT"
		Debuffs["growth-x"] = "RIGHT"
		Debuffs["growth-y"] = "DOWN"
		Debuffs["spacing-x"] = 3
		Debuffs:SetHeight(22)
		Debuffs:SetWidth(115)
		Debuffs.size = 22
		Debuffs.num = 5
		self.Debuffs = Debuffs

		Debuffs.PostUpdateIcon = PostUpdateIcon
	end,

	boss = function(self, ...)
		Shared(self, ...)

		local Health, Power = self.Health, self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		self:SetAttribute('initial-height', 14)
		self:SetAttribute('initial-width', 170)

		Health:SetHeight(12)

		local HealthPoints = F.CreateFS(Health, 8, "RIGHT")
		HealthPoints:SetPoint("RIGHT", self, "TOPRIGHT", 0, 6)
		self:Tag(HealthPoints, '[dead][free:health]')

		Health.value = HealthPoints

		local Name = F.CreateFS(Health, 8, "LEFT")
		Name:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 2)
		Name:SetWidth(110)
		Name:SetHeight(8)

		self:Tag(Name, '[name]')
		self.Name = Name

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		Castbar.Text = F.CreateFS(Health, 8)
		Castbar.Text:SetAllPoints(Health)

		local IconFrame = CreateFrame("Frame", nil, Castbar)
		IconFrame:SetPoint("LEFT", self, "RIGHT", 3, 0)
		IconFrame:SetHeight(22)
		IconFrame:SetWidth(22)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
		self.Iconbg:SetBackdrop({
			edgeFile = C.media.backdrop, 
			edgeSize = 1, 
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameLevel(IconFrame:GetFrameLevel()-1)

		local Buffs = CreateFrame("Frame", nil, self)
		Buffs.initialAnchor = "TOPLEFT"
		Buffs:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -4)
		Buffs["growth-x"] = "RIGHT"
		Buffs["growth-y"] = "DOWN"
		Buffs['spacing-x'] = 3
		Buffs['spacing-y'] = 3

		Buffs:SetHeight(22)
		Buffs:SetWidth(146)
		Buffs.num = 5
		Buffs.size = 26

		self.Buffs = Buffs

		Buffs.PostUpdateIcon = PostUpdateIcon

		local RaidIcon = F.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)
	end,

	arena = function(self, ...)
		Shared(self, ...)

		local Health, Power = self.Health, self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		self:SetAttribute('initial-height', 14)
		self:SetAttribute('initial-width', 170)

		Health:SetHeight(12)

		local HealthPoints = F.CreateFS(Health, 8, "RIGHT")
		HealthPoints:SetPoint("RIGHT", self, "TOPRIGHT", 0, 6)
		self:Tag(HealthPoints, '[dead][free:health]')

		Health.value = HealthPoints

		local Name = F.CreateFS(Health, 8, "LEFT")
		Name:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 2)
		Name:SetWidth(110)
		Name:SetHeight(8)

		self:Tag(Name, '[name]')
		self.Name = Name

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		Castbar.Text = F.CreateFS(Health, 8)
		Castbar.Text:SetAllPoints(Health)

		local IconFrame = CreateFrame("Frame", nil, Castbar)
		IconFrame:SetPoint("LEFT", self, "RIGHT", 3, 0)
		IconFrame:SetHeight(22)
		IconFrame:SetWidth(22)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
		self.Iconbg:SetBackdrop({
			edgeFile = C.media.backdrop, 
			edgeSize = 1, 
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameLevel(IconFrame:GetFrameLevel()-1)

		local Buffs = CreateFrame("Frame", nil, self)
		Buffs.initialAnchor = "TOPLEFT"
		Buffs:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -4)
		Buffs["growth-x"] = "RIGHT"
		Buffs["growth-y"] = "DOWN"
		Buffs['spacing-x'] = 3
		Buffs['spacing-y'] = 3

		Buffs:SetHeight(22)
		Buffs:SetWidth(146)
		Buffs.num = 5
		Buffs.size = 26

		self.Buffs = Buffs

		Buffs.PostUpdateIcon = PostUpdateIcon

		local RaidIcon = F.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		if(C.unitframes.arena_trinkets == true) then
			local Trinket = CreateFrame("Frame", nil, self)
			Trinket:SetHeight(22)
			Trinket:SetWidth(22)
			Trinket:SetPoint("RIGHT", self, "LEFT", -3, 0)
			Trinket.trinketUseAnnounce = true
			Trinket.trinketUpAnnounce = true

			self.Trinket = Trinket

			local f = CreateFrame("Frame", nil, Trinket)
			f:SetPoint("TOPLEFT", -1, 1)
			f:SetPoint("BOTTOMRIGHT", 1, -1)
			F.CreateBD(f, 0)
		end
	end,
}

do
	local range = {
		insideAlpha = 1,
		outsideAlpha = .5,
	}

	UnitSpecific.party = function(self, ...)
		Shared(self, ...)

		self.disallowVehicleSwap = false

		local Health, Power = self.Health, self.Power

		if roledb.role == 2 then
			Health:SetHeight(40)

			local Text = F.CreateFS(Health, 8, "CENTER")
			if C.unitframes.healmode_oldstyle == true then Text:SetPoint("CENTER", 0, 0) else Text:SetPoint("CENTER", 1, 0) end
			self.Text = Text

			self:Tag(Text, '[free:missinghealth]')
		else
			Health:SetHeight(20)

			local NotHere = F.CreateFS(Health, 8, "CENTER")
			NotHere:SetPoint("CENTER", 1, 0)

			self:Tag(NotHere, '[dead][offline]')
		end

		local RaidIcon = F.CreateFS(Health, 8)
		RaidIcon:SetPoint("CENTER")

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Leader = F.CreateFS(self, 8, "LEFT")
		Leader:SetText("l")
		Leader:SetPoint("TOPLEFT", Health, 1, -1)

		self.Leader = Leader

		local MasterLooter = F.CreateFS(self, 8, "RIGHT")
		MasterLooter:SetText("m")
		MasterLooter:SetPoint("TOPRIGHT", Health, 1, 0)

		self.MasterLooter = MasterLooter

		local picon = Health:CreateTexture(nil, "OVERLAY")
		picon:SetPoint("TOPRIGHT")
		picon:SetSize(8, 8)
		self.picon = PhaseIcon

		local rc = Health:CreateTexture(nil, "OVERLAY")
		rc:SetPoint"TOPLEFT"
		rc:SetHeight(16)
		rc:SetWidth(16)
		self.ReadyCheck = rc

		local UpdateLFD = function(self, event)
			local lfdrole = self.LFDRole
			local role = UnitGroupRolesAssigned(self.unit)

			if role == "DAMAGER" then
				lfdrole:SetTextColor(1, .1, .1, 1)
			elseif role == "TANK" then
				lfdrole:SetTextColor(.3, .4, 1, 1)
			elseif role == "HEALER" then
				lfdrole:SetTextColor(0, 1, 0, 1)
			else
				lfdrole:SetTextColor(0, 0, 0, 0)
			end
		end

		local lfd = F.CreateFS(Health, 16, "CENTER")
		lfd:SetPoint("BOTTOM", 1, -4)
		lfd:SetText("•")
		lfd.Override = UpdateLFD

		self.LFDRole = lfd

		local dbh = Health:CreateTexture(nil, "OVERLAY")
		dbh:SetTexture(C.media.questborder)
		dbh:SetVertexColor(0, 0, 0, 1)
		dbh.DebuffHighlightAlpha = 1
		dbh:SetBlendMode("ADD")
		dbh:SetPoint("TOPLEFT", self, -3, 3)
		dbh:SetPoint("BOTTOMRIGHT", self, 3, -3)
		self.DebuffHighlight = dbh
		self.DebuffHighlightFilter = true

		if(class == "DRUID" or class == "PALADIN" or class == "PRIEST" or class == "SHAMAN") and roledb.role == 2 then
			local auras = {}
			local spellIDs = { 
				--Druid spells 
				774, --Rejuvenation 
				8936, --Regrowth 
				33763, --Lifebloom 
				48438, --Wild Growth

				--Priest spells 
				6788,  --Weakened Soul 
				33076, --Prayer of Mending 
				139, --Renew 
				17, --Power Word: Shield

				--Shaman spells 
				61295, --Riptide  
				16236, --Ancestral Fortitude
				974, --Earth Shield

				--Paladin spells 
				53563, --Beacon of Light 
			}

			auras.onlyShowPresent = true
			auras.icons = {}
			for i, sid in pairs(spellIDs) do
				local icon = CreateFrame("Frame", nil, self)
				icon.spellID = sid
				icon:SetWidth(12)
				icon:SetHeight(12)

				if i == 1 or i == 5 or i == 9 or i == 12 then
					icon:SetPoint("BOTTOMLEFT", Health, "BOTTOMLEFT", 1, 1)
				elseif i == 2 or i == 6 or i == 10 then
					icon:SetPoint("BOTTOMRIGHT", Health, "BOTTOMRIGHT", -1, 1)
				elseif i == 3 or i == 7 or i == 11 then
					icon:SetPoint("TOPLEFT", Health, "TOPLEFT", 1, -1)
				elseif i == 4 or i == 8 then
					icon:SetPoint("TOPRIGHT", Health, "TOPRIGHT", -1, -1)
				end

				auras.icons[sid] = icon

				local cd = CreateFrame("Cooldown", nil, icon)
				cd:SetAllPoints(icon)
				cd:SetReverse()
				icon.cd = cd

				local count = F.CreateFS(icon, 8)
				count:SetPoint("TOP", icon, "TOP", 0, -2)
				icon.count = count

				F.CreateBD(icon)
			end
			self.AuraWatch = auras
		end

		if C.unitframes.raiddebuff == true and roledb.role == 2 then
			local Debuffs = CreateFrame("Frame", nil, self)
			Debuffs.initialAnchor = "CENTER"
			Debuffs:SetPoint("CENTER", self)
			Debuffs["growth-x"] = "LEFT"
			Debuffs["growth-y"] = "DOWN"
			Debuffs['spacing-x'] = 3
			Debuffs['spacing-y'] = 3

			Debuffs:SetHeight(22)
			Debuffs:SetWidth(22)
			Debuffs.num = 1
			Debuffs.size = 22

			self.Debuffs = Debuffs

			Debuffs.PostCreateIcon = function(icons, index)
				index:EnableMouse(false)
			end

			Debuffs.PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
				local name = UnitAura(unit, index, icon.filter)
				if(C.raidfilter[name]) then
					icon:Hide()
				else
					icon:Show()
				end
			end
		end

		local UpdateThreat = function(self, event, unit)
			if(unit ~= self.unit) then return end

			local threat = self.Threat

			unit = unit or self.unit
			local status = UnitThreatSituation(unit)

			if(status and status > 0) then
				local r, g, b = GetThreatStatusColor(status)
				threat:SetTextColor(r, g, b)
				threat:Show()
			else
				threat:Hide()
			end
		end

		local Threat = F.CreateFS(Health, 16)
		Threat:SetText("•")
		Threat:SetPoint("TOP", self, 1, 3)

		self.Threat = Threat
		Threat.Override = UpdateThreat

		self.Range = range
	end
end

--[[ Register and activate style ]]

oUF:RegisterStyle("Free", Shared)
for unit,layout in next, UnitSpecific do
	oUF:RegisterStyle('Free - ' .. unit:gsub("^%l", string.upper), layout)
end

local spawnHelper = function(self, unit, ...)
	if(UnitSpecific[unit]) then
		self:SetActiveStyle('Free - ' .. unit:gsub("^%l", string.upper))
	elseif(UnitSpecific[unit:match('[^%d]+')]) then -- boss1 -> boss
		self:SetActiveStyle('Free - ' .. unit:match('[^%d]+'):gsub("^%l", string.upper))
	else
		self:SetActiveStyle'Free'
	end

	local object = self:Spawn(unit)
	object:SetPoint(...)
	return object
end

oUF:Factory(function(self)
	spawnHelper(self, 'player', unpack(C.unitframes.player))
	spawnHelper(self, 'target', unpack(C.unitframes.target))

	if C.unitframes.playerframe_focus == true then
		spawnHelper(self, 'focus', "BOTTOMRIGHT", oUF_FreePlayer, "TOPRIGHT", 0, 12)
	else
		spawnHelper(self, 'focus', unpack(C.unitframes.focus))
	end
	if C.unitframes.playerframe_pet == true then
		spawnHelper(self, 'pet', "BOTTOMLEFT", oUF_FreePlayer, "TOPLEFT", 0, 12)
	else
		spawnHelper(self, 'pet', unpack(C.unitframes.pet))
	end

	for n = 1, 4 do
		spawnHelper(self,'boss' .. n, 'LEFT', 50, 0 - (56 * n))
	end

	for n = 1, 5 do
		spawnHelper(self, 'arena' .. n, 'LEFT', 50, 0 - (56 * n))
	end

	self:SetActiveStyle'Free - Party'

	local party_width, party_height
	if roledb.role == 2 then
		party_width = 42
		party_height = 42
	else
		party_width = 38
		party_height = 22
	end

	local party = self:SpawnHeader(nil, nil, 'party', 
		'showParty', true, 
		'showPlayer', true and roledb.role == 2 or false, 
		'showSolo', false, 
		'yoffset', -3, 
		'maxColumns', 5, 
		'unitsperColumn', 1, 
		'columnSpacing', 3, 
		'columnAnchorPoint', "RIGHT",
		'oUF-initialConfigFunction', ([[
			self:SetHeight(%d)
			self:SetWidth(%d)
		]]):format(party_height, party_width)
	)

	if roledb.role == 1 then
		party:SetPoint("BOTTOM", oUF_FreePlayer, "TOP", 0, 50)
	else
		party:SetPoint(unpack(C.unitframes.party))
	end

	local raid = self:SpawnHeader(nil, nil, 'raid',
		'showPlayer', true,
		'showParty', false,
		'showRaid', true,
		'xoffset', 5,
		'yOffset', -4,
		'point', "TOP",
		'groupFilter', '1,2,3,4,5,6,7,8',
		'groupingOrder', '1,2,3,4,5,6,7,8',
		'groupBy', 'GROUP',
		'maxColumns', 8,
		'unitsPerColumn', 5,
		'columnSpacing', 5,
		'columnAnchorPoint', "RIGHT",
		'oUF-initialConfigFunction', ([[
			self:SetHeight(%d)
			self:SetWidth(%d)
		]]):format(party_height, party_width)
	)

	if roledb.role == 1 then
		raid:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMLEFT", -5, 0)
	else
		raid:SetPoint(unpack(C.unitframes.raid))
	end
end)

--[[ Targettarget text ]]

local tt = CreateFrame("Frame", nil, UIParent)
local a1, p, a2, x, y = unpack(C.unitframes.target)
tt:SetPoint(a1, p, a2, x+60, y+26)
tt:SetWidth(110)
tt:SetHeight(12)

ttt = F.CreateFS(tt, 8, "RIGHT")
ttt:SetAllPoints(tt)

tt:RegisterEvent("UNIT_TARGET")
tt:RegisterEvent("PLAYER_TARGET_CHANGED")
tt:SetScript("OnEvent", function()
	if(UnitName("targettarget")==UnitName("player")) then
		ttt:SetText("> YOU <")
		ttt:SetTextColor(1, 0, 0)
	else
		ttt:SetText(UnitName"targettarget")
		ttt:SetTextColor(1, 1, 1)
	end
end)